const _ = require('lodash');
const chalk = require('chalk');
const constants = require('../generator-nodejs-constants');

const SERVER_NODEJS_DIR = `${constants.SERVER_NODEJS_SRC_DIR}/`;
const utils = require('../utils');

/**
 * The default is to use a file path string. It implies use of the template method.
 * For any other config an object { file:.., method:.., template:.. } can be used
 */
const serverFiles = {
    server: [
        {
            path: SERVER_NODEJS_DIR,
            templates: [
                {
                    file: 'src/repository/entity/entity.ts',
                    renameTo: generator => `src/repository/entity/${generator.entityFileName}.entity.ts`
                },
                {
                    file: 'src/repository/entity.repository.ts',
                    renameTo: generator => `src/repository/${generator.entityFileName}.repository.ts`
                },
                {
                    file: 'src/repository/mapper/entity.repository.mapper.ts',
                    renameTo: generator => `src/repository/mapper/${generator.entityFileName}.repository.mapper.ts`
                },
                {
                    file: 'src/module/entity.module.ts',
                    renameTo: generator => `src/module/${generator.entityFileName}.module.ts`
                },
                {
                    file: 'src/domain/interactors/entity.interactors.ts',
                    renameTo: generator => `src/domain/interactors/${generator.entityFileName}.interactor.ts`
                },
                {
                    file: 'src/domain/repository/entity.repository.ts',
                    renameTo: generator => `src/domain/repository/${generator.entityFileName}.repository.ts`
                },
                {
                    file: 'src/domain/model/entity.model.ts',
                    renameTo: generator => `src/domain/model/${generator.entityFileName}.model.ts`
                },
                {
                    file: 'src/api/dto/entity.dto.ts',
                    renameTo: generator => `src/api/dto/${generator.entityFileName}.dto.ts`
                },
                {
                    file: 'src/api/mapper/entity.mapper.ts',
                    renameTo: generator => `src/api/mapper/${generator.entityFileName}.mapper.ts`
                },
                {
                    file: 'src/api/rest/entity.controller.ts',
                    renameTo: generator => `src/api/rest/${generator.entityFileName}.controller.ts`
                },
                {
                    file: 'e2e/entity.e2e-spec.ts',
                    renameTo: generator => `e2e/${generator.entityFileName}.e2e-spec.ts`
                }
            ]
        }
    ]
};

module.exports = {
    writeFiles
};

function writeFiles() {
    return {
        customEntityServerFiles() {
            if (this.skipServer) return;

            if (this.databaseType === 'mongodb' && this.relationships.length > 0) {
                this.error(chalk.red('relationships not supported in mongodb!'));
            }

            this.writeFilesToDisk(serverFiles, this, false);

            this.fields.forEach(field => {
                if (field.fieldIsEnum === true) {
                    const enumFileName = _.kebabCase(field.fieldType);
                    const enumInfo = utils.buildEnumInfo(field, this.angularAppName, this.packageName, this.clientRootFolder);
                    this.template(
                        `${SERVER_NODEJS_DIR}src/repository/entity/enumeration/enum-type.ts.ejs`,
                        `${SERVER_NODEJS_DIR}src/repository/entity/enumeration/${enumFileName}.ts`,
                        this,
                        {},
                        enumInfo
                    );
                    this.template(
                        `${SERVER_NODEJS_DIR}src/domain/model/enumeration/enum-type.ts.ejs`,
                        `${SERVER_NODEJS_DIR}src/domain/model/enumeration/${enumFileName}.ts`,
                        this,
                        {},
                        enumInfo
                    );
                }
            });

            utils.addEntityToAppModuleImport(this, this.entityClass, this.entityFileName);
            utils.addEntityToAppModule(this, this.entityClass);
        }
    };
}
