const constants = require('../generator-nodejs-constants');

const SERVER_NODEJS_DIR = `${constants.SERVER_NODEJS_SRC_DIR}/`;

module.exports = {
    writeFiles
};

const serverFiles = {
    common: [
        {
            path: SERVER_NODEJS_DIR,
            templates: [
                'src/api/rest/user.controller.ts',
                'src/api/rest/account.controller.ts',
                'src/api/rest/public.user.controller.ts',
                'src/api/rest/management.controller.ts',
                'src/api/mapper/user.mapper.ts',
                'src/api/dto/base/pagination.dto.ts',
                'src/api/dto/base/base.dto.ts',
                'src/api/dto/password-change.dto.ts',
                'src/api/dto/user.dto.ts',
                'src/api/dto/user-login.dto.ts',
                'src/api/interceptors/unauthorized.ts',
                'src/repository/user.repository.ts',
                'src/domain/repository/base.repository.ts',
                'src/repository/role-permission.repository.ts',
                'src/repository/role.repository.ts',
                'src/module/user.module.ts',
                'src/module/auth.module.ts',
                'src/module/role-permission.module.ts',
                'src/module/role.module.ts',
                'src/config/application-dev.yml',
                'src/config/application-test.yml',
                'src/config.ts',
                'src/config/application-prod.yml',
                'src/config/application.yml',
                'src/config/constants.ts',
                'src/repository/entity/base/base.entity.ts',
                'src/repository/entity/user.entity.ts',
                'src/repository/entity/role.entity.ts',
                'src/repository/entity/role-permission.entity.ts',
                'src/security/guards/roles.guard.ts',
                'src/security/guards/auth.guard.ts',
                'src/security/role-type.ts',
                'src/security/password-util.ts',
                'src/security/decorators/auth-user.decorator.ts',
                'src/security/decorators/roles.decorator.ts',
                'src/security/index.ts',
                'src/client/header-util.ts',
                'src/client/interceptors/logging.interceptor.ts',
                'src/client/request.ts',
                'src/domain/interactors/auth.interactor.ts',
                'src/domain/interactors/user.interactor.ts',
                'src/domain/interactors/role-permission.interactor.ts',
                'src/domain/interactors/role.interactor.ts',
                'src/domain/model/base/base.model.ts',
                'src/domain/model/user.model.ts',
                'src/domain/model/role-permission.model.ts',
                'src/domain/model/role.model.ts',
                'src/domain/roles/model.conf',
                'src/domain/roles/role.manager.ts',
                'src/domain/errors/auth.errors.ts',
                'src/domain/errors/phone.verifier.errors.ts',
                'src/domain/jwt_generator/jwt.generator.ts',
                'src/domain/repository/user.repository.ts',
                'src/domain/repository/role.repository.ts',
                'src/domain/repository/role-permission.repository.ts',
                'src/providers/jwt_generator/jwt.service.ts',
                'src/repository/mapper/user.repository.mapper.ts',
                'src/repository/mapper/role-permission.repository.mapper.ts',
                'src/repository/mapper/role.repository.mapper.ts',
                'src/repository/mapper/user.repository.mapper.ts',
                'src/main.ts',
                'src/swagger.ts',
                'src/app.module.ts',
                'src/migrations/1570200270081-CreateTables.ts',
                'src/migrations/1570200320072-SeedRoles.ts',
                'src/migrations/1570200490072-SeedUsersRoles.ts',
                'src/orm.config.ts',
                'scripts/copy-resources.ts',
                'tsconfig.build.json',
                'test/admin/management.controller.spec.ts',
                'nest-cli.json',
                {
                    file: 'env',
                    renameTo: () => '.env'
                },
                {
                    file: 'server.eslintrc.json',
                    renameTo: () => '.server.eslintrc.json'
                },
                {
                    file: 'server.eslintignore',
                    renameTo: () => '.server.eslintignore'
                },
                {
                    file: 'prettierrc.js',
                    renameTo: () => '.prettierrc.js'
                },
                'package.json',
                'tsconfig.json',
                'README.md',
                'webpack.server.prod.config.js',
                'sonar-project.properties'
            ]
        }
    ],
    twilio: [
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withTwilio,
            templates: [
                'src/domain/interactors/phone.verifier.interactor.ts',
                'src/module/phone.verifier.module.ts',
                'src/domain/phone_verifier/phone.verifier.ts',
                'src/providers/phone_verifier/twilio.client.ts'
            ]
        }
    ],
    e2e: [
        {
            path: SERVER_NODEJS_DIR,
            templates: ['e2e/app.e2e-spec.ts', 'e2e/user.e2e-spec.ts', 'e2e/jest.e2e.config.json', 'e2e/setup.test.js']
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => !generator.skipUserManagement && generator.authenticationType !== 'oauth2',
            templates: ['e2e/account.e2e-spec.ts']
        }
    ],
    other: [
        {
            templates: [
                'package.json',
                'src/main/docker/app.yml',
                'src/main/docker/sonar.yml'
                /*
                    { file: '.mvn/wrapper/maven-wrapper.properties', method: 'copy', noEjs: true },
                */
            ]
        }
    ],
    jwt: [
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.authenticationType === 'jwt',
            templates: [
                'src/api/rest/user.jwt.controller.ts',
                'src/security/passport.jwt.strategy.ts',
                'src/security/payload.interface.ts',
                'src/domain/model/password-change.model.ts',
                'src/domain/model/user-login.model.ts'
            ]
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withGoogleAuth,
            templates: ['src/security/google.strategy.ts']
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withFacebookAuth,
            templates: ['src/security/facebook.strategy.ts']
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withTwitterAuth,
            templates: ['src/security/twitter.strategy.ts']
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withAppleAuth,
            templates: ['src/security/apple.strategy.ts', 'AuthKey_FUGV99L6VU.p8']
        },
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withMicrosoftAuth,
            templates: ['src/security/microsoft.strategy.ts']
        }
    ],
    oauth2: [
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.authenticationType === 'oauth2',
            templates: [
                'src/api/rest/user.oauth2.controller.ts',
                'src/security/passport.oauth2.strategy.ts',
                'src/security/oauth2.config.ts'
            ]
        }
    ],
    keycloak: [
        {
            condition: generator => generator.authenticationType === 'oauth2',
            templates: [
                'src/main/docker/keycloak.yml',
                'src/main/docker/realm-config/jhipster-realm.json',
                'src/main/docker/realm-config/jhipster-users-0.json'
            ]
        }
    ],
    db: [
        {
            condition: generator => generator.prodDatabaseType === 'mysql',
            templates: ['src/main/docker/mysql.yml']
        },
        {
            condition: generator => generator.prodDatabaseType === 'mssql',
            templates: ['src/main/docker/mssql.yml']
        },
        {
            condition: generator => generator.prodDatabaseType === 'postgresql',
            templates: ['src/main/docker/postgresql.yml']
        },
        {
            condition: generator => generator.prodDatabaseType === 'mongodb',
            templates: ['src/main/docker/mongodb.yml']
        }
    ],
    graphql: [
        {
            path: SERVER_NODEJS_DIR,
            condition: generator => generator.withGraphQL,
            templates: [
                'src/api/dto/base/pagination-graphql/pagination-args.ts',
                'src/domain/jwt_generator/graphql-jwt-auth.guard.ts',
                'src/api/dto/create-user.input.ts',
                'src/api/dto/update-user.input.ts',
                'src/api/graphql/resolvers/user.resolver.ts'
            ]
        }
    ]
};

function writeFiles() {
    return {
        writeSameFiles() {
            this.writeFilesToDisk(serverFiles, this, false);
        }
    };
}
